@@ -15,12 +15,13 @@ The following functions are exported
 =cut
 
 use strict;
-use vars qw(@ISA @EXPORT $VERSION);
+use vars qw(@ISA @EXPORT $VERSION @EXPORT_OK);
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(TTF_Init_Fields TTF_Read_Fields TTF_Out_Fields TTF_Pack
              TTF_Unpack TTF_word_utf8 TTF_utf8_word);
+@EXPORT_OK = (@EXPORT, qw(XML_hexdump));
 $VERSION = 0.0001;
 
 =head2 ($val, $pos) = TTF_Init_Fields ($str, $pos)
@@ -233,12 +234,12 @@ sub TTF_Pack
             }
             elsif ($type =~ m/^[l]/oi)
             {
-                $res += 1 << 32 if $res < 0;
+                $res += 1 << 32 if ($type eq 'L' && $res < 0);
                 $dat .= pack("N", $res);
             }
             elsif ($type =~ m/^[s]/oi)
             {
-                $res += 1 << 16 if $res < 0;
+                $res += 1 << 16 if ($type eq 'S' && $res < 0);
                 $dat .= pack("n", $res);
             }
             elsif ($type eq "c")
@@ -286,7 +287,7 @@ sub TTF_word_utf8
     my ($res, $i);
     my (@dat) = unpack("n*", $str);
 
-    return pack("U*", @dat) if ($^V ge v5.6.0);
+    return pack("U*", @dat) if ($^V && $^V ge v5.6.0);
     for ($i = 0; $i <= $#dat; $i++)
     {
         my ($dat) = $dat[$i];
@@ -347,7 +348,141 @@ sub TTF_utf8_word
     }
     $res;
 }
-            
+
+
+=head2 XML_hexdump($context, $dat)
+
+Dumps out the given data as a sequence of <data> blocks each 16 bytes wide
+
+=cut
+
+sub XML_hexdump
+{
+    my ($context, $depth, $dat) = @_;
+    my ($fh) = $context->{'fh'};
+    my ($i, $len, $out);
+
+    $len = length($dat);
+    for ($i = 0; $i < $len; $i += 16)
+    {
+        $out = join(' ', map {sprintf("%02X", ord($_))} (split('', substr($dat, $i, 16))));
+        $fh->printf("%s<data addr='%04X'>%s</data>\n", $depth, $i, $out);
+    }
+}
+
+
+=head2 XML_outhints
+
+Converts a binary string of hinting code into a textual representation
+
+=cut
+
+{
+    my (@hints) = (
+    ['SVTCA[0]'], ['SVTCA[1]'], ['SPVTCA[0]'], ['SPVTCA[1]'], ['SFVTCA[0]'], ['SFVTCA[1]'], ['SPVTL[0]'], ['SPVTL[1]'],
+    ['SFVTL[0]'], ['SFVTL[1]'], ['SPVFS'], ['SFVFS'], ['GPV'], ['GFV'], ['SVFTPV'], ['ISECT'],
+# 10
+    ['SRP0'], ['SRP1'], ['SRP2'], ['SZP0'], ['SZP1'], ['SZP2'], ['SZPS'], ['SLOOP'],
+    ['RTG'], ['RTHG'], ['SMD'], ['ELSE'], ['JMPR'], ['SCVTCI'], ['SSWCI'], ['SSW'],
+# 20
+    ['DUP'], ['POP'], ['CLEAR'], ['SWAP'], ['DEPTH'], ['CINDEX'], ['MINDEX'], ['ALIGNPTS'],
+    [], ['UTP'], ['LOOPCALL'], ['CALL'], ['FDEF'], ['ENDF'], ['MDAP[0]'], ['MDAP[1]'],
+# 30
+    ['IUP[0]'], ['IUP[1]'], ['SHP[0]'], ['SHP[1]'], ['SHC[0]'], ['SHC[1]'], ['SHZ[0]'], ['SHZ[1]'],
+    ['SHPIX'], ['IP'], ['MSIRP[0]'], ['MSIRP[1]'], ['ALIGNRP'], ['RTDG'], ['MIAP[0]'], ['MIAP[1]'],
+# 40
+    ['NPUSHB', -1, 1], ['NPUSHW', -1, 2], ['WS', 0, 0], ['RS', 0, 0], ['WCVTP', 0, 0], ['RCVT', 0, 0], ['GC[0]'], ['GC[1]'],
+    ['SCFS'], ['MD[0]'], ['MD[1]'], ['MPPEM'], ['MPS'], ['FLIPON'], ['FLIPOFF'], ['DEBUG'],
+# 50
+    ['LT'], ['LTEQ'], ['GT'], ['GTEQ'], ['EQ'], ['NEQ'], ['ODD'], ['EVEN'],
+    ['IF'], ['EIF'], ['AND'], ['OR'], ['NOT'], ['DELTAP1'], ['SDB'], ['SDS'],
+# 60
+    ['ADD'], ['SUB'], ['DIV'], ['MULT'], ['ABS'], ['NEG'], ['FLOOR'], ['CEILING'],
+    ['ROUND[0]'], ['ROUND[1]'], ['ROUND[2]'], ['ROUND[3]'], ['NROUND[0]'], ['NROUND[1]'], ['NROUND[2]'], ['NROUND[3]'],
+# 70
+    ['WCVTF'], ['DELTAP2'], ['DELTAP3'], ['DELTAC1'], ['DELTAC2'], ['DELTAC3'], ['SROUND'], ['S45ROUND'],
+    ['JROT'], ['JROF'], ['ROFF'], [], ['RUTG'], ['RDTG'], ['SANGW'], [],
+# 80
+    ['FLIPPT'], ['FLIPRGON'], ['FLIPRGOFF'], [], [], ['SCANCTRL'], ['SDPVTL[0]'], ['SDPVTL[1]'],
+    ['GETINFO'], ['IDEF'], ['ROLL'], ['MAX'], ['MIN'], ['SCANTYPE'], ['INSTCTRL'], [],
+# 90
+    [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [],
+# A0
+    [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [],
+# B0
+    ['PUSHB1', 1, 1], ['PUSHB2', 2, 1], ['PUSHB3', 3, 1], ['PUSHB4', 4, 1], ['PUSHB5', 5, 1], ['PUSHB6', 6, 1], ['PUSHB7', 7, 1], ['PUSHB8', 8, 1],
+    ['PUSHW1', 1, 2], ['PUSHW2', 2, 2], ['PUSHW3', 3, 2], ['PUSHW4', 4, 2], ['PUSHW5', 5, 2], ['PUSHW6', 6, 2], ['PUSHW7', 7, 2], ['PUSHW8', 8, 2],
+# C0
+    ['MDRP[0]'], ['MDRP[1]'], ['MDRP[2]'], ['MDRP[3]'], ['MDRP[4]'], ['MDRP[5]'], ['MDRP[6]'], ['MDRP[7]'],
+    ['MDRP[8]'], ['MDRP[9]'], ['MDRP[A]'], ['MDRP[B]'], ['MDRP[C]'], ['MDRP[D]'], ['MDRP[E]'], ['MDRP[F]'],
+# D0
+    ['MDRP[10]'], ['MDRP[11]'], ['MDRP[12]'], ['MDRP[13]'], ['MDRP[14]'], ['MDRP[15]'], ['MDRP[16]'], ['MDRP[17]'],
+    ['MDRP[18]'], ['MDRP[19]'], ['MDRP[1A]'], ['MDRP[1B]'], ['MDRP[1C]'], ['MDRP[1D]'], ['MDRP[1E]'], ['MDRP[1F]'],
+# E0
+    ['MIRP[0]'], ['MIRP[1]'], ['MIRP[2]'], ['MIRP[3]'], ['MIRP[4]'], ['MIRP[5]'], ['MIRP[6]'], ['MIRP[7]'],
+    ['MIRP[8]'], ['MIRP[9]'], ['MIRP[A]'], ['MIRP[B]'], ['MIRP[C]'], ['MIRP[D]'], ['MIRP[E]'], ['MIRP[F]'],
+# F0
+    ['MIRP[10]'], ['MIRP[11]'], ['MIRP[12]'], ['MIRP[13]'], ['MIRP[14]'], ['MIRP[15]'], ['MIRP[16]'], ['MIRP[17]'],
+    ['MIRP[18]'], ['MIRP[19]'], ['MIRP[1A]'], ['MIRP[1B]'], ['MIRP[1C]'], ['MIRP[1D]'], ['MIRP[1E]'], ['MIRP[1F]']);
+
+    my ($i);
+    my (%hints) = map { $_->[0] => $i++ } @hints;
+
+    sub XML_binhint
+    {
+        my ($dat) = @_;
+        my ($len) = length($dat);
+        my ($res, $i);
+        my ($text, $num, $size);
+
+        for ($i = 0; $i < $len; $i++)
+        {
+            ($text, $num, $size) = @{$hints[ord(substr($dat, $i, 1))]};
+            $text = sprintf("UNK[%02X]", ord(substr($dat, $i, 1))) unless defined $text;
+            $res .= $text;
+            if ($num != 0)
+            {
+                if ($num < 0)
+                {
+                    $i++;
+                    my ($nnum) = unpack($num == -1 ? 'C' : 'n', substr($dat, $i, -$num));
+                    $i += -$num - 1;
+                    $num = $nnum;
+                }
+                $res .= "\t" . join(' ', unpack($size == 1 ? 'C*' : 'n*', substr($dat, $i + 1, $num * $size)));
+                $i += $num * $size;
+            }
+            $res .= "\n";
+        }
+        $res;
+    }
+
+    sub XML_hintbin
+    {
+        my ($dat) = @_;
+        my ($l, $res, @words, $num);
+
+        foreach $l (split(/\s*\n\s*/, $dat))
+        {
+            @words = split(/\s*/, $l);
+            next unless (defined $hints{$words[0]});
+            $num = $hints{$words[0]};
+            $res .= pack('C', $num);
+            if ($hints[$num][1] < 0)
+            {
+                $res .= pack($hints[$num][1] == -1 ? 'C' : 'n', $#words);
+                $res .= pack($hints[$num][2] == 1 ? 'C*' : 'n*', @words[1 .. $#words]);
+            }
+            elsif ($hints[$num][1] > 0)
+            {
+                $res .= pack($hints[$num][2] == 1 ? 'C*' : 'n*', @words[1 .. $hints[$num][1]]);
+            }
+        }
+        $res;
+    }
+}
+
+
 1;
 
 =head1 BUGS
